var Relationship = require('./Relationship');

var QueryHelper = {
  table: function(table, attribute) {
    var raw = '"' + table + '"';
    if(attribute) {
      raw += '.' + '"' + attribute + '"';
    }
    return raw;
  },
  value: function(value) {
    return '\'' + value + '\'';
  },
  select: function(select, from) {
    var raw = 'select ';
    select.forEach(function(s, index) {
      raw += this.table(s[0], s[1]);
      if(select.length > index + 1) {
        raw += ' ,';
      }
    }, this);
    raw += ' from ' + this.table(from);
    return raw;
  },
  where: function(whereOperator, table, attribute, operator, value) {
    return whereOperator + ' ' + this.table(table, attribute) + ' ' + operator + ' ' + this.value(value);
  },
  whereSelect: function(whereOperator, ta, aa, tb, ab, tc, ac, td, ad, te, ae, o, v) {
    var raw = whereOperator + ' ' + this.table(ta, aa) + ' in (';
    raw += this.select([[tb, ab]], tb);
    raw += ' ' + this.join(tc, td, ac, '=', ad);
    raw += ' ' + this.where('where', te, ae, o, v);
    raw += ')';
    return raw;
  },
  join: function(joinTable, currentTable, joinTableIdentifer, operator, currentTableIdentifer) {
    return 'inner join '+this.table(joinTable)+' on '+this.table(joinTable, joinTableIdentifer)+' ' + operator + ' ' + this.table(currentTable, currentTableIdentifer);
  },
  buildJoinRaw: function(tableName, join, count, relationships) {
    var raw = ''
    for(var relationshipTableName in join) {
      var relationship = relationships[relationshipTableName];
      if(relationship.type === Relationship.Types.belongsToMany) {
        if(raw.length) {
          raw += ' ';
        }
        raw += this.join(relationship.throughTableName, tableName, relationship.identifer.parent_0, '=', relationship.identifer.child_0);
        count++;
      }
      else {
        if(raw.length) {
          raw += ' ';
        }
        raw += this.join(relationship.modelFactoryName, tableName, relationship.identifer.parent, '=', relationship.identifer.child);
        count++;
      }
    }
    return {
      raw: raw,
      count: count
    };
  },
  buildWhereRaw: function(tableName, where, count, relationships) {
    var raw = '';
    for(var whereAttribute in where) {
      if(raw.length) {
        raw += ' ';
      }
      var whereOperator = (count) ? 'and' : 'where';
      raw += this.where(whereOperator, tableName, whereAttribute, '=', where[whereAttribute]);
      count++;
    }
    return {
      raw: raw,
      count: count
    };
  },
  buildWhereSelectRaw: function(table, where, count, relationship) {
    var raw = '';
    where.forEach(function(w) {
      for(var attribute in w) {
        var whereOperator = (count) ? 'and' : 'where';
        raw += this.whereSelect(whereOperator,
          table,
          relationship.identifer.child_0,
          relationship.throughTableName,
          relationship.identifer.parent_0,
          relationship.modelFactoryName,
          relationship.identifer.child_1,
          relationship.throughTableName,
          relationship.identifer.parent_1,
          relationship.modelFactoryName,
          attribute,
          '=',
          w[attribute]
        );
        count++;
      }
    }, this);
    return {
      raw: raw,
      count: count
    };
  },
  buildSelectRaw: function(tableName, select) {
    var raw = 'select distinct ';
    var count = 0;
    select.forEach(function(attribute) {
      if(count) {
        raw += ', ';
      }
      raw += this.table(tableName, attribute);
      count++;
    }, this);
    raw += ' from ' + '"'+tableName+'"';
    return {
      raw: raw
    };
  }
};

var Query = {
  create: function(ORM, tableName, data, returnAttributes) {
    return ORM.knex(tableName).insert(data, returnAttributes);
  },
  update: function(ORM, tableName, where, data, returnAttributes) {
    return ORM.knex(tableName).where(where).update(data, returnAttributes);
  },
  find: function(ORM, tableName, where, returnAttributes) {
    return ORM.knex(tableName).where(where).select(returnAttributes);
  },
  findJoin: function(ORM, tableName, join, where, relationships, returnAttributes) {

    var selectRaw = QueryHelper.buildSelectRaw(tableName, returnAttributes).raw;
    var joinRaw = '';

    // INNER JOIN
    var joinCount = 0;
    var joinResults = QueryHelper.buildJoinRaw(tableName, join, joinCount, relationships);
    joinCount += joinResults.count;
    joinRaw += joinResults.raw;

    // WHERE
    var whereRaw = '';
    var whereCount = 0;

    // Table Where
    var whereResults = QueryHelper.buildWhereRaw(tableName, where, whereCount, relationships);
    whereCount += whereResults.count;
    whereRaw += ' ' + whereResults.raw;

    // JOIN WHERE
    for(var relationshipTableName in join) {
      if(relationships.hasOwnProperty(relationshipTableName)) {
        var relationship = relationships[relationshipTableName];
        switch(relationship.type) {
          case Relationship.Types.belongsToOne: {
            var joinWhereResults = QueryHelper.buildWhereRaw(relationshipTableName, join[relationshipTableName], whereCount);
            whereCount += joinWhereResults.count;
            whereRaw += ' ' + joinWhereResults.raw;
            break;
          }
          case Relationship.Types.belongsToMany: {
            var joinWhereResults = QueryHelper.buildWhereSelectRaw(tableName, join[relationshipTableName], whereCount, relationship);
            whereCount += joinWhereResults.count;
            whereRaw += ' ' + joinWhereResults.raw;
            break;
          }
          case Relationship.Types.hasOne: {
            break;
          }
          case Relationship.Types.hasMany: {
            break;
          }
        }
      }
    }

    var raw = '';
    raw += selectRaw;
    raw += ' ' + joinRaw;
    raw += whereRaw;

    return ORM.knex.raw(raw, []).then(function(res) {
      return res.rows;
    });
  },
  delete: function(ORM, tableName, where) {
    return ORM.knex(tableName).where(where, false).del();
  },
};

module.exports = Query;