var Relationship = require('./Relationship');

function QueryObject() {
  this.attributes = {};
  this.relationships = {
    belongsToOne: {},
    belongsToMany: {},
    hasOne: {},
    hasMany: {}
  };
  return this;
}

QueryObject.prototype.filter = function(attributes, relationships) {
  for(var attribute in attributes) {
    if(relationships.hasOwnProperty(attribute)) {
      // Sort the relationships
      switch(relationships[attribute].type) {
        case Relationship.Types.belongsToOne: {
          this.relationships.belongsToOne[attribute] = attributes[attribute];
          break;
        }
        case Relationship.Types.belongsToMany: {
          this.relationships.belongsToMany[attribute] = attributes[attribute];
          break;
        }
        case Relationship.Types.hasOne: {
          this.relationships.hasOne[attribute] = attributes[attribute];
          break;
        }
        case Relationship.Types.hasMany: {
          this.relationships.hasMany[attribute] = attributes[attribute];
          break;
        }
      }
    } else {
      this.attributes[attribute] = attributes[attribute];
    }
  }
  return this;
};

//
// Relationships
//

QueryObject.prototype.attachBelongsToOneRelationships = function(relationships) {
  for(var attribute in this.relationships.belongsToOne) {
    if(relationships.hasOwnProperty(attribute)) {
      this.attributes = relationships[attribute].attachAsAttribute(this.attributes, this.relationships.belongsToOne[attribute]);
    }
  }
  return this;
};

QueryObject.prototype.attachBelongsToManyRelationships = function(childModel, relationships) {
  var promises = [];
  for(var attribute in this.relationships.belongsToMany) {
    if(relationships.hasOwnProperty(attribute)) {
      var promise = relationships[attribute].attach(childModel, this.relationships.belongsToMany[attribute]);
      promises.push(promise);
    }
  }
  return Promise.all(promises).then(function(values) {
    return childModel;
  });
};

//
// Getters
//

QueryObject.prototype.getAttributes = function() {
  return this.attributes;
};

QueryObject.prototype.getRelationships = function() {
  return Object.assign({},
    this.relationships.belongsToOne,
    this.relationships.belongsToMany,
    this.relationships.hasOne,
    this.relationships.hasMany);
};

module.exports = QueryObject;