exports.up = function(knex) {
  return knex.raw('CREATE EXTENSION IF NOT EXISTS "uuid-ossp"')
  .then(function() {
    return knex.schema
    .createTable('user', function(table) {
      table.uuid('id').primary().defaultTo(knex.raw('uuid_generate_v4()'));
      table.string('email').notNullable().unique();
      table.string('username').notNullable().unique();
      table.string('password').notNullable();
      table.string('team');
      table.boolean('private');
      table.dateTime('created_at').defaultTo(knex.fn.now());
      table.dateTime('updated_at').defaultTo(knex.fn.now());
    })
    .createTable('tag', function(table) {
      table.uuid('id').primary().defaultTo(knex.raw('uuid_generate_v4()'));
      table.string('name').notNullable().unique();
      table.dateTime('created_at').defaultTo(knex.fn.now());
      table.dateTime('updated_at').defaultTo(knex.fn.now());
    })
    .createTable('post', function(table) {
      table.uuid('id').primary().defaultTo(knex.raw('uuid_generate_v4()'));
      table.string('title').notNullable();
      table.string('content');
      table.uuid('user_id').notNullable().references('user.id').onDelete('CASCADE');
      table.dateTime('created_at').defaultTo(knex.fn.now());
      table.dateTime('updated_at').defaultTo(knex.fn.now());
    })
    .createTable('post_tag', function(table) {
      table.uuid('post_id').references('post.id').onDelete('CASCADE');
      table.uuid('tag_id').references('tag.id').onDelete('CASCADE');
    });
  });
};

exports.down = function(knex) {
  return knex.schema
  .dropTable('post_tag')
  .dropTable('post')
  .dropTable('tag')
  .dropTable('user');
};
