'use strict';
require('./bootstrap');

describe('ORM', function() {

  var tags, user;

  // Create post with user
  it('should create a user', function(done) {
    ORM.models.user.create({
      email: 'test@gmail.com',
      username: 'test',
      password: 'test123',
      team: 'red',
      private: false
    })
    .then(function(model) {
      model.email.should.equal('test@gmail.com');
      model.username.should.equal('test');
      model.team.should.equal('red');
      model.should.not.have.property('password');
      user = model;
      done();
    })
    .catch(function(error) {
      console.log(error);
    });
  });

  it('should create tags', function(done) {
    Promise.all([
      ORM.models.tag.create({
        name: 'red',
      }),
      ORM.models.tag.create({
        name: 'blue',
      }),
      ORM.models.tag.create({
        name: 'green',
      })
    ])
    .then(function(models) {
      models[0].name.should.equal('red');
      models[1].name.should.equal('blue');
      models[2].name.should.equal('green');
      tags = models;
      done();
    })
    .catch(function(error) {
      console.log(error);
    });
  });

  it('create post with a user and tag relation', function(done) {
    Promise.all([
      ORM.models.post.create({
        title: 'Hello World',
        content: 'Lorem ipsum',
        tags: [tags[0], tags[1], tags[2]],
        user: user
      }),
      ORM.models.post.create({
        title: 'This is a test',
        content: 'Lorem ipsum',
        tags: [tags[0], tags[1]],
        user: user
      })
    ])
    .then(function(models) {
      models[0].title.should.equal('Hello World');
      models[0].content.should.equal('Lorem ipsum');
      models[1].title.should.equal('This is a test');
      models[1].content.should.equal('Lorem ipsum');
      done();
    })
    .catch(function(error) {
      console.log(error);
    });
  });

  it('should find created post with user relation', function(done) {
    ORM.models.post.find({
      user: {
        private: false
      },
      tags: [
        { name: 'red' },
        { name: 'blue' }
      ]
    }, {
      include: ['user', 'tags']
    })
    .then(function(models) {
      models.should.have.length(2);
      models[0].should.have.property('user');
      models[1].should.have.property('user');
      models[0].should.have.property('tags');
      models[1].should.have.property('tags');
      done();
    })
    .catch(function(error) {
      console.log(error);
    });
  });

  it('should find created post with user relation', function(done) {
    ORM.models.post.find({
      user: {
        private: false
      },
      tags: [
        { name: 'red' },
        { name: 'blue' },
        { name: 'green' }
      ]
    }, {
      include: ['user', 'tags']
    })
    .then(function(models) {
      models.should.have.length(1);
      models[0].should.have.property('user');
      models[0].should.have.property('tags');
      done();
    })
    .catch(function(error) {
      console.log(error);
    });
  });

});